@@ -22,11 +22,7 @@ import           GHC.Settings
 
 -- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
 
-#if !MIN_VERSION_ghc(9,3,0)
-import qualified GHC.Driver.Pipeline             as Pipeline
-#endif
-
-#if MIN_VERSION_ghc(9,3,0) && !MIN_VERSION_ghc(9,5,0)
+#if !MIN_VERSION_ghc(9,5,0)
 import qualified GHC.Driver.Pipeline.Execute     as Pipeline
 #endif
 
@@ -38,6 +34,10 @@ import qualified GHC.SysTools.Cpp                as Pipeline
 import qualified GHC.SysTools.Tasks              as Pipeline
 #endif
 
+#if MIN_VERSION_ghc(9,11,0)
+import qualified GHC.SysTools.Tasks              as Pipeline
+#endif
+
 addOptP :: String -> DynFlags -> DynFlags
 addOptP f = alterToolSettings $ \s -> s
           { toolSettings_opt_P             = f : toolSettings_opt_P s
