@@ -58,9 +58,6 @@ module Development.IDE.GHC.Compat.Core (
     pattern ExposePackage,
     parseDynamicFlagsCmdLine,
     parseDynamicFilePragma,
-#if !MIN_VERSION_ghc(9,3,0)
-    WarnReason(..),
-#endif
     wWarningFlags,
     updOptLevel,
     -- slightly unsafe
@@ -72,12 +69,14 @@ module Development.IDE.GHC.Compat.Core (
     IfaceTyCon(..),
     ModIface,
     ModIface_(..),
+#if MIN_VERSION_ghc(9,11,0)
+    pattern ModIface,
+    set_mi_top_env,
+    set_mi_usages,
+#endif
     HscSource(..),
     WhereFrom(..),
     loadInterface,
-#if !MIN_VERSION_ghc(9,3,0)
-    SourceModified(..),
-#endif
     loadModuleInterface,
     RecompileRequired(..),
     mkPartialIface,
@@ -236,7 +235,11 @@ module Development.IDE.GHC.Compat.Core (
     ModuleOrigin(..),
     PackageName(..),
     -- * Linker
+#if MIN_VERSION_ghc(9,11,0)
+    LinkablePart(..),
+#else
     Unlinked(..),
+#endif
     Linkable(..),
     unload,
     -- * Hooks
@@ -359,7 +362,6 @@ module Development.IDE.GHC.Compat.Core (
     module GHC.Parser.Header,
     module GHC.Parser.Lexer,
     module GHC.Utils.Panic,
-#if MIN_VERSION_ghc(9,3,0)
     CompileReason(..),
     hsc_type_env_vars,
     hscUpdateHUG, hsc_HUG,
@@ -372,7 +374,6 @@ module Development.IDE.GHC.Compat.Core (
     module GHC.Unit.Finder.Types,
     module GHC.Unit.Env,
     module GHC.Driver.Phases,
-#endif
 #if !MIN_VERSION_ghc(9,4,0)
     pattern HsFieldBind,
     hfbAnn,
@@ -460,7 +461,7 @@ import           GHC.Tc.Types.Evidence       hiding ((<.>))
 import           GHC.Tc.Utils.Env
 import           GHC.Tc.Utils.Monad          hiding (Applicative (..), IORef,
                                               MonadFix (..), MonadIO (..), allM,
-                                              anyM, concatMapM, mapMaybeM,
+                                              anyM, concatMapM, mapMaybeM, foldMapM,
                                               (<$>))
 import           GHC.Tc.Utils.TcType         as TcType
 import qualified GHC.Types.Avail             as Avail
@@ -495,12 +496,15 @@ import qualified GHC.Utils.Panic.Plain       as Plain
 import           Data.Foldable               (toList)
 import           GHC.Core.Multiplicity       (scaledThing)
 import           GHC.Data.Bag
+import qualified GHC.Data.Strict             as Strict
+import qualified GHC.Driver.Config.Finder    as GHC
+import qualified GHC.Driver.Config.Tidy      as GHC
 import           GHC.Driver.Env
-import           GHC.Hs                       (HsModule (..))
-#if !MIN_VERSION_ghc(9,9,0)
-import           GHC.Hs                       (SrcSpanAnn')
-#endif
-import           GHC.Hs.Decls                 hiding (FunDep)
+import           GHC.Driver.Env              as GHCi
+import           GHC.Driver.Env.KnotVars
+import           GHC.Driver.Errors.Types
+import           GHC.Hs                      (HsModule (..))
+import           GHC.Hs.Decls                hiding (FunDep)
 import           GHC.Hs.Doc
 import           GHC.Hs.Expr
 import           GHC.Hs.Extension
@@ -524,73 +528,66 @@ import           GHC.Types.SourceText
 import           GHC.Types.Target            (Target (..), TargetId (..))
 import           GHC.Types.TyThing
 import           GHC.Types.TyThing.Ppr
-import           GHC.Unit.Finder             hiding (mkHomeModLocation)
-import           GHC.Unit.Home.ModInfo
-import           GHC.Unit.Module.Imported
-import           GHC.Unit.Module.ModDetails
-import           GHC.Unit.Module.ModGuts
-import           GHC.Unit.Module.ModIface    (IfaceExport, ModIface,
-                                              ModIface_ (..), mi_fix)
-import           GHC.Unit.Module.ModSummary  (ModSummary (..))
-import           Language.Haskell.Syntax     hiding (FunDep)
-
--- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
-
-#if !MIN_VERSION_ghc(9,3,0)
-import           GHC.Types.SourceFile        (SourceModified (..))
-import qualified GHC.Unit.Finder             as GHC
-import           GHC.Unit.Module.Graph       (mkModuleGraph)
-#endif
-
-#if MIN_VERSION_ghc(9,3,0)
-import qualified GHC.Data.Strict             as Strict
-import qualified GHC.Driver.Config.Finder    as GHC
-import qualified GHC.Driver.Config.Tidy      as GHC
-import           GHC.Driver.Env              as GHCi
-import           GHC.Driver.Env.KnotVars
-import           GHC.Driver.Errors.Types
 import           GHC.Types.Unique
 import           GHC.Types.Unique.Map
 import           GHC.Unit.Env
+import           GHC.Unit.Finder             hiding (mkHomeModLocation)
 import qualified GHC.Unit.Finder             as GHC
 import           GHC.Unit.Finder.Types
+import           GHC.Unit.Home.ModInfo
 import           GHC.Unit.Module.Graph
+import           GHC.Unit.Module.Imported
+import           GHC.Unit.Module.ModDetails
+import           GHC.Unit.Module.ModGuts
+#if !MIN_VERSION_ghc(9,9,0)
+import           GHC.Hs                      (SrcSpanAnn')
+#endif
+import           GHC.Unit.Module.ModIface    (IfaceExport, ModIface,
+                                              ModIface_ (..), mi_fix
+#if MIN_VERSION_ghc(9,11,0)
+                                             , pattern ModIface
+                                             , set_mi_top_env
+                                             , set_mi_usages
+#endif
+                                             )
+import           GHC.Unit.Module.ModSummary  (ModSummary (..))
 import           GHC.Utils.Error             (mkPlainErrorMsgEnvelope)
 import           GHC.Utils.Panic
 import           GHC.Utils.TmpFs
+import           Language.Haskell.Syntax     hiding (FunDep)
+
+
+-- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
+
+#if MIN_VERSION_ghc(9,11,0)
+import System.OsPath
 #endif
 
 #if !MIN_VERSION_ghc(9,7,0)
 import           GHC.Types.Avail             (greNamePrintableName)
 #endif
 
+#if !MIN_VERSION_ghc(9,9,0)
+import           GHC.Hs                      (SrcSpanAnn')
+#endif
+
 mkHomeModLocation :: DynFlags -> ModuleName -> FilePath -> IO Module.ModLocation
-#if MIN_VERSION_ghc(9,3,0)
+#if MIN_VERSION_ghc(9,11,0)
+mkHomeModLocation df mn f =
+  let osf = unsafeEncodeUtf f
+  in pure $ GHC.mkHomeModLocation (GHC.initFinderOpts df) mn osf
+#else
 mkHomeModLocation df mn f = pure $ GHC.mkHomeModLocation (GHC.initFinderOpts df) mn f
-#else
-mkHomeModLocation = GHC.mkHomeModLocation
 #endif
 
-#if MIN_VERSION_ghc(9,3,0)
 pattern RealSrcSpan :: SrcLoc.RealSrcSpan -> Maybe BufSpan -> SrcLoc.SrcSpan
-#else
-pattern RealSrcSpan :: SrcLoc.RealSrcSpan -> Maybe BufSpan -> SrcLoc.SrcSpan
-#endif
 
-#if MIN_VERSION_ghc(9,3,0)
 pattern RealSrcSpan x y <- SrcLoc.RealSrcSpan x ((\case Strict.Nothing -> Nothing; Strict.Just a -> Just a) -> y) where
   RealSrcSpan x y = SrcLoc.RealSrcSpan x (case y of Nothing -> Strict.Nothing; Just a -> Strict.Just a)
 
-#else
-pattern RealSrcSpan x y = SrcLoc.RealSrcSpan x y
-#endif
 {-# COMPLETE RealSrcSpan, UnhelpfulSpan #-}
 
-#if MIN_VERSION_ghc(9,3,0)
 pattern RealSrcLoc :: SrcLoc.RealSrcLoc -> Strict.Maybe BufPos-> SrcLoc.SrcLoc
-#else
-pattern RealSrcLoc :: SrcLoc.RealSrcLoc -> Maybe BufPos-> SrcLoc.SrcLoc
-#endif
 pattern RealSrcLoc x y = SrcLoc.RealSrcLoc x y
 {-# COMPLETE RealSrcLoc, UnhelpfulLoc #-}
 
@@ -664,6 +661,8 @@ instance HasSrcSpan (EpAnn a) where
 #if MIN_VERSION_ghc(9,9,0)
 instance HasSrcSpan (SrcLoc.GenLocated (EpAnn ann) a) where
   getLoc (L l _) = getLoc l
+instance HasSrcSpan (SrcLoc.GenLocated (GHC.EpaLocation) a) where
+  getLoc = GHC.getHasLoc
 #else
 instance HasSrcSpan (SrcSpanAnn' ann) where
   getLoc = GHC.locA
@@ -703,7 +702,7 @@ initObjLinker env =
 loadDLL :: HscEnv -> String -> IO (Maybe String)
 loadDLL env str = do
     res <- GHCi.loadDLL (GHCi.hscInterp env) str
-#if MIN_VERSION_ghc(9,11,0)
+#if MIN_VERSION_ghc(9,11,0) || (MIN_VERSION_ghc(9, 8, 3) && !MIN_VERSION_ghc(9, 9, 0))
     pure $
       case res of
         Left err_msg -> Just err_msg
@@ -718,12 +717,6 @@ unload hsc_env linkables =
     (GHCi.hscInterp hsc_env)
     hsc_env linkables
 
-#if !MIN_VERSION_ghc(9,3,0)
-setOutputFile :: FilePath -> DynFlags -> DynFlags
-setOutputFile f d = d {
-  outputFile_    = Just f
-  }
-#endif
 
 isSubspanOfA :: LocatedAn la a -> LocatedAn lb b -> Bool
 isSubspanOfA a b = SrcLoc.isSubspanOf (GHC.getLocA a) (GHC.getLocA b)
@@ -744,7 +737,7 @@ pattern GRE{gre_name, gre_par, gre_lcl, gre_imp} <- RdrName.GRE
 #endif
     ,gre_par, gre_lcl, gre_imp = (toList -> gre_imp)}
 
-collectHsBindsBinders :: CollectPass p => Bag (XRec p (HsBindLR p idR)) -> [IdP p]
+collectHsBindsBinders :: CollectPass p => LHsBindsLR p idR -> [IdP p]
 collectHsBindsBinders x = GHC.collectHsBindsBinders CollNoDictBinders x
 
 
@@ -752,54 +745,28 @@ collectHsBindsBinders x = GHC.collectHsBindsBinders CollNoDictBinders x
 makeSimpleDetails :: HscEnv -> TcGblEnv -> IO ModDetails
 makeSimpleDetails hsc_env =
   GHC.makeSimpleDetails
-#if MIN_VERSION_ghc(9,3,0)
               (hsc_logger hsc_env)
-#else
-              hsc_env
-#endif
 
 mkIfaceTc :: HscEnv -> GHC.SafeHaskellMode -> ModDetails -> ModSummary -> Maybe CoreProgram -> TcGblEnv -> IO ModIface
 mkIfaceTc hscEnv shm md _ms _mcp =
 #if MIN_VERSION_ghc(9,5,0)
   GHC.mkIfaceTc hscEnv shm md _ms _mcp -- mcp::Maybe CoreProgram is only used in GHC >= 9.6
-#elif MIN_VERSION_ghc(9,3,0)
-  GHC.mkIfaceTc hscEnv shm md _ms -- ms::ModSummary is only used in GHC >= 9.4
 #else
-  GHC.mkIfaceTc hscEnv shm md
+  GHC.mkIfaceTc hscEnv shm md _ms -- ms::ModSummary is only used in GHC >= 9.4
 #endif
 
 mkBootModDetailsTc :: HscEnv -> TcGblEnv -> IO ModDetails
 mkBootModDetailsTc session = GHC.mkBootModDetailsTc
-#if MIN_VERSION_ghc(9,3,0)
           (hsc_logger session)
-#else
-          session
-#endif
 
-#if !MIN_VERSION_ghc(9,3,0)
-type TidyOpts = HscEnv
-#endif
 
 initTidyOpts :: HscEnv -> IO TidyOpts
 initTidyOpts =
-#if MIN_VERSION_ghc(9,3,0)
   GHC.initTidyOpts
-#else
-  pure
-#endif
 
-#if MIN_VERSION_ghc(9,3,0)
 driverNoStop :: StopPhase
 driverNoStop = NoStop
-#else
-driverNoStop :: Phase
-driverNoStop = StopLn
-#endif
 
-#if !MIN_VERSION_ghc(9,3,0)
-hscUpdateHPT :: (HomePackageTable -> HomePackageTable) -> HscEnv -> HscEnv
-hscUpdateHPT k session = session { hsc_HPT = k (hsc_HPT session) }
-#endif
 
 #if !MIN_VERSION_ghc(9,4,0)
 pattern HsFieldBind :: XHsRecField id -> id -> arg -> Bool -> HsRecField' id arg
@@ -846,11 +813,7 @@ field_label = id
 #endif
 
 mkSimpleTarget :: DynFlags -> FilePath -> Target
-#if MIN_VERSION_ghc(9,3,0)
 mkSimpleTarget df fp = Target (TargetFile fp Nothing) True (homeUnitId_ df) Nothing
-#else
-mkSimpleTarget _ fp = Target (TargetFile fp Nothing) True Nothing
-#endif
 
 #if MIN_VERSION_ghc(9,7,0)
 lookupGlobalRdrEnv gre_env occ = lookupGRE gre_env (LookupOccName occ AllRelevantGREs)
