@@ -5,22 +5,20 @@
 module Development.IDE.GHC.Compat.Parser (
     initParserOpts,
     initParserState,
-    ApiAnns,
     PsSpan(..),
     pattern HsParsedModule,
     type GHC.HsParsedModule,
     Development.IDE.GHC.Compat.Parser.hpm_module,
     Development.IDE.GHC.Compat.Parser.hpm_src_files,
-    Development.IDE.GHC.Compat.Parser.hpm_annotations,
     pattern ParsedModule,
     Development.IDE.GHC.Compat.Parser.pm_parsed_source,
     type GHC.ParsedModule,
     Development.IDE.GHC.Compat.Parser.pm_mod_summary,
     Development.IDE.GHC.Compat.Parser.pm_extra_src_files,
-    Development.IDE.GHC.Compat.Parser.pm_annotations,
-    mkApiAnns,
     -- * API Annotations
+#if !MIN_VERSION_ghc(9,11,0)
     Anno.AnnKeywordId(..),
+#endif
     pattern EpaLineComment,
     pattern EpaBlockComment
     ) where
@@ -38,17 +36,8 @@ import           GHC                             (EpaCommentTok (..),
                                                   pm_mod_summary,
                                                   pm_parsed_source)
 import qualified GHC
-import           GHC.Hs                          (hpm_module, hpm_src_files)
-
--- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
-
-#if !MIN_VERSION_ghc(9,3,0)
-import qualified GHC.Driver.Config               as Config
-#endif
-
-#if MIN_VERSION_ghc(9,3,0)
 import qualified GHC.Driver.Config.Parser        as Config
-#endif
+import           GHC.Hs                          (hpm_module, hpm_src_files)
 
 
 
@@ -60,34 +49,28 @@ initParserState :: ParserOpts -> StringBuffer -> RealSrcLoc -> PState
 initParserState =
   Lexer.initParserState
 
--- GHC 9.2 does not have ApiAnns anymore packaged in ParsedModule. Now the
--- annotations are found in the ast.
-type ApiAnns = ()
-
 #if MIN_VERSION_ghc(9,5,0)
-pattern HsParsedModule :: Located (HsModule GhcPs) -> [FilePath] -> ApiAnns -> GHC.HsParsedModule
+pattern HsParsedModule :: Located (HsModule GhcPs) -> [FilePath] -> GHC.HsParsedModule
 #else
-pattern HsParsedModule :: Located HsModule -> [FilePath] -> ApiAnns -> GHC.HsParsedModule
+pattern HsParsedModule :: Located HsModule -> [FilePath] -> GHC.HsParsedModule
 #endif
 pattern HsParsedModule
     { hpm_module
     , hpm_src_files
-    , hpm_annotations
-    } <- ( (,()) -> (GHC.HsParsedModule{..}, hpm_annotations))
+    } <- GHC.HsParsedModule{..}
     where
-        HsParsedModule hpm_module hpm_src_files _hpm_annotations =
+        HsParsedModule hpm_module hpm_src_files =
             GHC.HsParsedModule hpm_module hpm_src_files
 
 
-pattern ParsedModule :: ModSummary -> ParsedSource -> [FilePath] -> ApiAnns -> GHC.ParsedModule
+pattern ParsedModule :: ModSummary -> ParsedSource -> [FilePath] -> GHC.ParsedModule
 pattern ParsedModule
     { pm_mod_summary
     , pm_parsed_source
     , pm_extra_src_files
-    , pm_annotations
-    } <- ( (,()) -> (GHC.ParsedModule{..}, pm_annotations))
+    } <- GHC.ParsedModule{..}
     where
-        ParsedModule ms parsed extra_src_files _anns =
+        ParsedModule ms parsed extra_src_files =
             GHC.ParsedModule
              { pm_mod_summary = ms
              , pm_parsed_source = parsed
@@ -95,6 +78,4 @@ pattern ParsedModule
              }
 {-# COMPLETE ParsedModule :: GHC.ParsedModule #-}
 
-mkApiAnns :: PState -> ApiAnns
-mkApiAnns = const ()
 
