@@ -7,7 +7,9 @@
 -- | Orphan instances for GHC.
 --   Note that the 'NFData' instances may not be law abiding.
 module Development.IDE.GHC.Orphans() where
-import           Development.IDE.GHC.Compat
+import           Development.IDE.GHC.Compat        hiding
+                                                   (DuplicateRecordFields,
+                                                    FieldSelectors)
 import           Development.IDE.GHC.Util
 
 import           Control.DeepSeq
@@ -17,27 +19,19 @@ import           Data.Hashable
 import           Data.String                       (IsString (fromString))
 import           Data.Text                         (unpack)
 
+import           Data.Bifunctor                    (Bifunctor (..))
 import           GHC.ByteCode.Types
 import           GHC.Data.Bag
 import           GHC.Data.FastString
 import qualified GHC.Data.StringBuffer             as SB
+import           GHC.Parser.Annotation
+import           GHC.Types.FieldLabel              (DuplicateRecordFields (DuplicateRecordFields, NoDuplicateRecordFields),
+                                                    FieldSelectors (FieldSelectors, NoFieldSelectors))
+import           GHC.Types.PkgQual
 import           GHC.Types.SrcLoc
 
 -- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
 
-#if !MIN_VERSION_ghc(9,3,0)
-import           GHC.Types.Unique                  (getKey)
-import           GHC.Unit.Module.Graph             (ModuleGraph)
-#endif
-
-import           Data.Bifunctor                    (Bifunctor (..))
-import           GHC.Parser.Annotation
-
-#if MIN_VERSION_ghc(9,3,0)
-import           GHC.Types.PkgQual
-
-#endif
-
 #if MIN_VERSION_ghc(9,5,0)
 import           GHC.Unit.Home.ModInfo
 import           GHC.Unit.Module.Location          (ModLocation (..))
@@ -57,6 +51,17 @@ instance Show ModDetails where show = const "<moddetails>"
 instance NFData ModDetails where rnf = rwhnf
 instance NFData SafeHaskellMode where rnf = rwhnf
 instance Show Linkable where show = unpack . printOutputable
+#if MIN_VERSION_ghc(9,11,0)
+instance NFData Linkable where rnf (Linkable a b c) = rnf a `seq` rnf b `seq` rnf c
+instance NFData LinkableObjectSort where rnf = rwhnf
+instance NFData LinkablePart where
+  rnf (DotO a b)         = rnf a `seq` rnf b
+  rnf (DotA f)           = rnf f
+  rnf (DotDLL f)         = rnf f
+  rnf (BCOs a)           = seqCompiledByteCode a
+  rnf (CoreBindings wcb) = rnf wcb
+  rnf (LazyBCOs a b)     = seqCompiledByteCode a `seq` rnf b
+#else
 instance NFData Linkable where rnf (LM a b c) = rnf a `seq` rnf b `seq` rnf c
 instance NFData Unlinked where
   rnf (DotO f)           = rnf f
@@ -66,13 +71,23 @@ instance NFData Unlinked where
 #if MIN_VERSION_ghc(9,5,0)
   rnf (CoreBindings wcb) = rnf wcb
   rnf (LoadedBCOs us)    = rnf us
+#endif
+#endif
 
+#if MIN_VERSION_ghc(9,5,0)
 instance NFData WholeCoreBindings where
+#if MIN_VERSION_ghc(9,11,0)
+  rnf (WholeCoreBindings bs m ml f) = rnf bs `seq` rnf m `seq` rnf ml `seq` rnf f
+#else
   rnf (WholeCoreBindings bs m ml) = rnf bs `seq` rnf m `seq` rnf ml
+#endif
 
 instance NFData ModLocation where
+#if MIN_VERSION_ghc(9,11,0)
+    rnf (OsPathModLocation mf f1 f2 f3 f4 f5) = rnf mf `seq` rnf f1 `seq` rnf f2 `seq` rnf f3 `seq` rnf f4 `seq` rnf f5
+#else
     rnf (ModLocation mf f1 f2 f3 f4 f5) = rnf mf `seq` rnf f1 `seq` rnf f2 `seq` rnf f3 `seq` rnf f4 `seq` rnf f5
-
+#endif
 #endif
 
 instance Show PackageFlag where show = unpack . printOutputable
@@ -88,9 +103,6 @@ instance NFData SB.StringBuffer where rnf = rwhnf
 instance Show Module where
     show = moduleNameString . moduleName
 
-#if !MIN_VERSION_ghc(9,3,0)
-instance Outputable a => Show (GenLocated SrcSpan a) where show = unpack . printOutputable
-#endif
 
 #if !MIN_VERSION_ghc(9,5,0)
 instance (NFData l, NFData e) => NFData (GenLocated l e) where
@@ -131,12 +143,6 @@ instance Show HieFile where
 instance NFData HieFile where
     rnf = rwhnf
 
-#if !MIN_VERSION_ghc(9,3,0)
-deriving instance Eq SourceModified
-deriving instance Show SourceModified
-instance NFData SourceModified where
-    rnf = rwhnf
-#endif
 
 instance Hashable ModuleName where
     hashWithSalt salt = hashWithSalt salt . show
@@ -222,7 +228,6 @@ instance NFData ModuleGraph where rnf = rwhnf
 instance NFData HomeModInfo where
   rnf (HomeModInfo iface dets link) = rwhnf iface `seq` rnf dets `seq` rnf link
 
-#if MIN_VERSION_ghc(9,3,0)
 instance NFData PkgQual where
   rnf NoPkgQual      = ()
   rnf (ThisPkg uid)  = rnf uid
@@ -233,7 +238,6 @@ instance NFData UnitId where
 
 instance NFData NodeKey where
   rnf = rwhnf
-#endif
 
 #if MIN_VERSION_ghc(9,5,0)
 instance NFData HomeModLinkable where
@@ -246,8 +250,27 @@ instance NFData (HsExpr (GhcPass Renamed)) where
 instance NFData (Pat (GhcPass Renamed)) where
     rnf = rwhnf
 
+instance NFData (HsExpr (GhcPass Typechecked)) where
+    rnf = rwhnf
+
+instance NFData (Pat (GhcPass Typechecked)) where
+    rnf = rwhnf
+
 instance NFData Extension where
   rnf = rwhnf
 
 instance NFData (UniqFM Name [Name]) where
   rnf (ufmToIntMap -> m) = rnf m
+
+#if !MIN_VERSION_ghc(9,5,0)
+instance NFData DuplicateRecordFields where
+  rnf DuplicateRecordFields   = ()
+  rnf NoDuplicateRecordFields = ()
+
+instance NFData FieldSelectors where
+  rnf FieldSelectors   = ()
+  rnf NoFieldSelectors = ()
+
+instance NFData FieldLabel where
+  rnf (FieldLabel a b c d) = rnf a `seq` rnf b `seq` rnf c `seq` rnf d
+#endif
