@@ -20,6 +20,7 @@ module Development.IDE.Import.DependencyInformation
   , insertImport
   , pathToId
   , idToPath
+  , idToModLocation
   , reachableModules
   , processDependencyInformation
   , transitiveDeps
@@ -47,21 +48,14 @@ import           Data.List.NonEmpty                 (NonEmpty (..), nonEmpty)
 import qualified Data.List.NonEmpty                 as NonEmpty
 import           Data.Maybe
 import           Data.Tuple.Extra                   hiding (first, second)
+import           Development.IDE.GHC.Compat
 import           Development.IDE.GHC.Orphans        ()
-import           GHC.Generics                       (Generic)
-import           Prelude                            hiding (mod)
-
 import           Development.IDE.Import.FindImports (ArtifactsLocation (..))
 import           Development.IDE.Types.Diagnostics
 import           Development.IDE.Types.Location
+import           GHC.Generics                       (Generic)
+import           Prelude                            hiding (mod)
 
-import           Development.IDE.GHC.Compat
-
--- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
-
-#if !MIN_VERSION_ghc(9,3,0)
-import           GHC.Unit.Module.Graph              (ModuleGraph)
-#endif
 
 -- | The imports for a given module.
 newtype ModuleImports = ModuleImports
