@@ -14,29 +14,23 @@ module Development.IDE.Import.FindImports
   ) where
 
 import           Control.DeepSeq
-import           Development.IDE.GHC.Compat        as Compat
-import           Development.IDE.GHC.Error         as ErrUtils
-import           Development.IDE.GHC.Orphans       ()
-import           Development.IDE.Types.Diagnostics
-import           Development.IDE.Types.Location
-
--- standard imports
 import           Control.Monad.Extra
 import           Control.Monad.IO.Class
 import           Data.List                         (find, isSuffixOf)
 import           Data.Maybe
 import qualified Data.Set                          as S
-import           System.FilePath
-
--- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
-
-#if !MIN_VERSION_ghc(9,3,0)
-import           Development.IDE.GHC.Compat.Util
-#endif
-
-#if MIN_VERSION_ghc(9,3,0)
+import           Development.IDE.GHC.Compat        as Compat
+import           Development.IDE.GHC.Error         as ErrUtils
+import           Development.IDE.GHC.Orphans       ()
+import           Development.IDE.Types.Diagnostics
+import           Development.IDE.Types.Location
 import           GHC.Types.PkgQual
 import           GHC.Unit.State
+import           System.FilePath
+
+
+#if MIN_VERSION_ghc(9,11,0)
+import           GHC.Driver.DynFlags
 #endif
 
 data Import
@@ -105,12 +99,11 @@ locateModuleFile import_dirss exts targetFor isSource modName = do
 -- It only returns Just for unit-ids which are possible to import into the
 -- current module. In particular, it will return Nothing for 'main' components
 -- as they can never be imported into another package.
-#if MIN_VERSION_ghc(9,3,0)
 mkImportDirs :: HscEnv -> (UnitId, DynFlags) -> Maybe (UnitId, ([FilePath], S.Set ModuleName))
-mkImportDirs _env (i, flags) = Just (i, (importPaths flags, reexportedModules flags))
+#if MIN_VERSION_ghc(9,11,0)
+mkImportDirs _env (i, flags) = Just (i, (importPaths flags, S.fromList $ map reexportTo $ reexportedModules flags))
 #else
-mkImportDirs :: HscEnv -> (UnitId, DynFlags) -> Maybe (PackageName, (UnitId, [FilePath], S.Set ModuleName))
-mkImportDirs env (i, flags) = (, (i, importPaths flags, S.empty)) <$> getUnitName env i
+mkImportDirs _env (i, flags) = Just (i, (importPaths flags, reexportedModules flags))
 #endif
 
 -- | locate a module in either the file system or the package database. Where we go from *daml to
@@ -122,42 +115,22 @@ locateModule
     -> [String]                        -- ^ File extensions
     -> (ModuleName -> NormalizedFilePath -> m (Maybe NormalizedFilePath))  -- ^ does file exist predicate
     -> Located ModuleName              -- ^ Module name
-#if MIN_VERSION_ghc(9,3,0)
     -> PkgQual                -- ^ Package name
-#else
-    -> Maybe FastString                -- ^ Package name
-#endif
     -> Bool                            -- ^ Is boot module
     -> m (Either [FileDiagnostic] Import)
 locateModule env comp_info exts targetFor modName mbPkgName isSource = do
   case mbPkgName of
-#if MIN_VERSION_ghc(9,3,0)
     -- 'ThisPkg' just means some home module, not the current unit
     ThisPkg uid
       | Just (dirs, reexports) <- lookup uid import_paths
           -> lookupLocal uid dirs reexports
       | otherwise -> return $ Left $ notFoundErr env modName $ LookupNotFound []
-#else
-    -- "this" means that we should only look in the current package
-    Just "this" -> do
-      lookupLocal (homeUnitId_ dflags) (importPaths dflags) S.empty
-#endif
     -- if a package name is given we only go look for a package
-#if MIN_VERSION_ghc(9,3,0)
     OtherPkg uid
       | Just (dirs, reexports) <- lookup uid import_paths
           -> lookupLocal uid dirs reexports
-#else
-    Just pkgName
-      | Just (uid, dirs, reexports) <- lookup (PackageName pkgName) import_paths
-          -> lookupLocal uid dirs reexports
-#endif
       | otherwise -> lookupInPackageDB
-#if MIN_VERSION_ghc(9,3,0)
     NoPkgQual -> do
-#else
-    Nothing -> do
-#endif
 
       -- Reexports for current unit have to be empty because they only apply to other units depending on the
       -- current unit. If we set the reexports to be the actual reexports then we risk looping forever trying
@@ -181,7 +154,11 @@ locateModule env comp_info exts targetFor modName mbPkgName isSource = do
       -- about which module unit a imports.
       -- Without multi-component support it is hard to recontruct the dependency environment so
       -- unit a will have both unit b and unit c in scope.
+#if MIN_VERSION_ghc(9,11,0)
+      map (\uid -> let this_df = homeUnitEnv_dflags (ue_findHomeUnitEnv uid ue) in (uid, importPaths this_df, S.fromList $ map reexportTo $ reexportedModules this_df)) hpt_deps
+#else
       map (\uid -> let this_df = homeUnitEnv_dflags (ue_findHomeUnitEnv uid ue) in (uid, importPaths this_df, reexportedModules this_df)) hpt_deps
+#endif
     ue = hsc_unit_env env
     units = homeUnitEnv_units $ ue_findHomeUnitEnv (homeUnitId_ dflags) ue
     hpt_deps :: [UnitId]
@@ -196,11 +173,7 @@ locateModule env comp_info exts targetFor modName mbPkgName isSource = do
       -- This is particularly important for Paths_* modules which get generated for every component but unless you use it in
       -- each component will end up being found in the wrong place and cause a multi-cradle match failure.
     _import_paths' = -- import_paths' is only used in GHC < 9.4
-#if MIN_VERSION_ghc(9,3,0)
             import_paths
-#else
-            map snd import_paths
-#endif
 
     toModLocation uid file = liftIO $ do
         loc <- mkHomeModLocation dflags (unLoc modName) (fromNormalizedFilePath file)
@@ -226,7 +199,7 @@ notFoundErr env modName reason =
   mkError' $ ppr' $ cannotFindModule env modName0 $ lookupToFindResult reason
   where
     dfs = hsc_dflags env
-    mkError' = diagFromString "not found" DiagnosticSeverity_Error (Compat.getLoc modName)
+    mkError' doc = diagFromString "not found" DiagnosticSeverity_Error (Compat.getLoc modName) doc Nothing
     modName0 = unLoc modName
     ppr' = showSDoc dfs
     -- We convert the lookup result to a find result to reuse GHC's cannotFindModule pretty printer.
@@ -263,10 +236,5 @@ notFound = NotFound
   , fr_suggestions = []
   }
 
-#if MIN_VERSION_ghc(9,3,0)
 noPkgQual :: PkgQual
 noPkgQual = NoPkgQual
-#else
-noPkgQual :: Maybe a
-noPkgQual = Nothing
-#endif
