@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP              #-}
 {-# LANGUAGE DeriveAnyClass   #-}
 {-# LANGUAGE OverloadedLabels #-}
 {-# LANGUAGE TypeFamilies     #-}
@@ -26,7 +27,8 @@ import           Data.List                            (find)
 import qualified Data.Map                             as Map
 import           Data.Maybe                           (catMaybes, maybeToList)
 import qualified Data.Text                            as T
-import           Development.IDE                      (GhcSession (..),
+import           Development.IDE                      (FileDiagnostic (..),
+                                                       GhcSession (..),
                                                        HscEnvEq (hscEnv),
                                                        RuleResult, Rules, Uri,
                                                        define, srcSpanToRange,
@@ -126,9 +128,10 @@ codeLensProvider ideState pId CodeLensParams{_textDocument = TextDocumentIdentif
           -- We don't actually pass any data to resolve, however we need this
           -- dummy type to make sure HLS resolves our lens
           [ CodeLens _range Nothing (Just $ toJSON TypeLensesResolve)
-            | (dFile, _, diag@Diagnostic{_range}) <- diags
-            , dFile == nfp
-            , isGlobalDiagnostic diag]
+            | diag <- diags
+            , let lspDiag@Diagnostic {_range} = fdLspDiagnostic diag
+            , fdFilePath diag == nfp
+            , isGlobalDiagnostic lspDiag]
         -- The second option is to generate lenses from the GlobalBindingTypeSig
         -- rule. This is the only type that needs to have the range adjusted
         -- with PositionMapping.
@@ -319,7 +322,11 @@ gblBindingType (Just hsc) (Just gblEnv) = do
         let name = idName identifier
         hasSig name $ do
           env <- tcInitTidyEnv
+#if MIN_VERSION_ghc(9,11,0)
+          let ty = tidyOpenType env (idType identifier)
+#else
           let (_, ty) = tidyOpenType env (idType identifier)
+#endif
           pure $ GlobalBindingTypeSig name (printName name <> " :: " <> showDoc (pprSigmaType ty)) (name `elemNameSet` exports)
       patToSig p = do
         let name = patSynName p
