@@ -1,25 +1,21 @@
+{-# LANGUAGE CPP #-}
 module Development.IDE.Types.HscEnvEq
 (   HscEnvEq,
     hscEnv, newHscEnvEq,
-    hscEnvWithImportPaths,
-    newHscEnvEqPreserveImportPaths,
-    newHscEnvEqWithImportPaths,
     updateHscEnvEq,
-    envImportPaths,
     envPackageExports,
     envVisibleModuleNames,
-    deps
 ) where
 
 
 import           Control.Concurrent.Async        (Async, async, waitCatch)
 import           Control.Concurrent.Strict       (modifyVar, newVar)
-import           Control.DeepSeq                 (force)
+import           Control.DeepSeq                 (force, rwhnf)
 import           Control.Exception               (evaluate, mask, throwIO)
 import           Control.Monad.Extra             (eitherM, join, mapMaybeM)
 import           Data.Either                     (fromRight)
-import           Data.Set                        (Set)
-import qualified Data.Set                        as Set
+import           Data.IORef
+import qualified Data.Map                        as M
 import           Data.Unique                     (Unique)
 import qualified Data.Unique                     as Unique
 import           Development.IDE.GHC.Compat      hiding (newUnique)
@@ -28,9 +24,11 @@ import           Development.IDE.GHC.Error       (catchSrcErrors)
 import           Development.IDE.GHC.Util        (lookupPackageConfig)
 import           Development.IDE.Graph.Classes
 import           Development.IDE.Types.Exports   (ExportsMap, createExportsMap)
+import           GHC.Driver.Env                  (hsc_all_home_unit_ids)
 import           Ide.PluginUtils                 (toAbsolute)
 import           OpenTelemetry.Eventlog          (withSpan)
-import           System.FilePath
+import           System.Directory                (makeAbsolute)
+
 
 -- | An 'HscEnv' with equality. Two values are considered equal
 --   if they are created with the same call to 'newHscEnvEq' or
@@ -38,13 +36,6 @@ import           System.FilePath
 data HscEnvEq = HscEnvEq
     { envUnique             :: !Unique
     , hscEnv                :: !HscEnv
-    , deps                  :: [(UnitId, DynFlags)]
-               -- ^ In memory components for this HscEnv
-               -- This is only used at the moment for the import dirs in
-               -- the DynFlags
-    , envImportPaths        :: Maybe (Set FilePath)
-        -- ^ If Just, import dirs originally configured in this env
-        --   If Nothing, the env import dirs are unaltered
     , envPackageExports     :: IO ExportsMap
     , envVisibleModuleNames :: IO (Maybe [ModuleName])
         -- ^ 'listVisibleModuleNames' is a pure function,
@@ -59,18 +50,32 @@ updateHscEnvEq oldHscEnvEq newHscEnv = do
   update <$> Unique.newUnique
 
 -- | Wrap an 'HscEnv' into an 'HscEnvEq'.
-newHscEnvEq :: FilePath -> FilePath -> HscEnv -> [(UnitId, DynFlags)] -> IO HscEnvEq
-newHscEnvEq root cradlePath hscEnv0 deps = do
-    let relativeToCradle = (takeDirectory cradlePath </>)
-        hscEnv = removeImportPaths hscEnv0
+newHscEnvEq :: HscEnv -> IO HscEnvEq
+newHscEnvEq hscEnv' = do
 
-    -- Make Absolute since targets are also absolute
-    let importPathsCanon = toAbsolute root . relativeToCradle <$> importPaths (hsc_dflags hscEnv0)
+    mod_cache <- newIORef emptyInstalledModuleEnv
+    file_cache <- newIORef M.empty
+    -- This finder cache is for things which are outside of things which are tracked
+    -- by HLS. For example, non-home modules, dependent object files etc
+#if MIN_VERSION_ghc(9,11,0)
+    let hscEnv = hscEnv'
+               { hsc_FC = FinderCache
+                        { flushFinderCaches = \_ -> error "GHC should never call flushFinderCaches outside the driver"
+                        , addToFinderCache  = \(GWIB im _) val -> do
+                            if moduleUnit im `elem` hsc_all_home_unit_ids hscEnv'
+                            then error "tried to add home module to FC"
+                            else atomicModifyIORef' mod_cache $ \c -> (extendInstalledModuleEnv c im val, ())
+                        , lookupFinderCache = \(GWIB im _) -> do
+                            if moduleUnit im `elem` hsc_all_home_unit_ids hscEnv'
+                            then error ("tried to lookup home module from FC" ++ showSDocUnsafe (ppr (im, hsc_all_home_unit_ids hscEnv')))
+                            else lookupInstalledModuleEnv <$> readIORef mod_cache <*> pure im
+                        , lookupFileCache = \fp -> error ("not used by HLS" ++ fp)
+                        }
+                }
 
-    newHscEnvEqWithImportPaths (Just $ Set.fromList importPathsCanon) hscEnv deps
-
-newHscEnvEqWithImportPaths :: Maybe (Set FilePath) -> HscEnv -> [(UnitId, DynFlags)] -> IO HscEnvEq
-newHscEnvEqWithImportPaths envImportPaths hscEnv deps = do
+#else
+    let hscEnv = hscEnv'
+#endif
 
     let dflags = hsc_dflags hscEnv
 
@@ -112,23 +117,6 @@ newHscEnvEqWithImportPaths envImportPaths hscEnv deps = do
 
     return HscEnvEq{..}
 
--- | Wrap an 'HscEnv' into an 'HscEnvEq'.
-newHscEnvEqPreserveImportPaths
-    :: HscEnv -> [(UnitId, DynFlags)] -> IO HscEnvEq
-newHscEnvEqPreserveImportPaths = newHscEnvEqWithImportPaths Nothing
-
--- | Unwrap the 'HscEnv' with the original import paths.
---   Used only for locating imports
-hscEnvWithImportPaths :: HscEnvEq -> HscEnv
-hscEnvWithImportPaths HscEnvEq{..}
-    | Just imps <- envImportPaths
-    = hscSetFlags (setImportPaths (Set.toList imps) (hsc_dflags hscEnv)) hscEnv
-    | otherwise
-    = hscEnv
-
-removeImportPaths :: HscEnv -> HscEnv
-removeImportPaths hsc = hscSetFlags (setImportPaths [] (hsc_dflags hsc)) hsc
-
 instance Show HscEnvEq where
   show HscEnvEq{envUnique} = "HscEnvEq " ++ show (Unique.hashUnique envUnique)
 
@@ -136,9 +124,9 @@ instance Eq HscEnvEq where
   a == b = envUnique a == envUnique b
 
 instance NFData HscEnvEq where
-  rnf (HscEnvEq a b c d _ _) =
+  rnf (HscEnvEq a b _ _) =
       -- deliberately skip the package exports map and visible module names
-      rnf (Unique.hashUnique a) `seq` b `seq` c `seq` rnf d
+      rnf (Unique.hashUnique a) `seq` rwhnf b
 
 instance Hashable HscEnvEq where
   hashWithSalt s = hashWithSalt s . envUnique
